// *******************************************************************
// Broker.cpp
// *******************************************************************
//
//  DESCRIPTION: API for Broker class. Used by clients of the
//                        DocConversion project to broker a conversion
//                        of a given document.
//  NOTE:            This program makes use of strings, streams and
//                       Broker.h.
//
/***************************************************************************
*   Copyright (C) 2003 by drs. Eric D. Schabell                           *
*   erics@cs.kun.nl                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <iostream>          // file accessing.
#include <string>            // STL strings.
#include <pqxx/all.h>        // postgresql api.
#include "Broker.h"
#include "Server.h"

using namespace std;
using namespace pqxx;


//--------------------------------------------------------------------
//Variables
//--------------------------------------------------------------------
int Broker::brokerCounter           = 0;
const string Broker::DEFAULT_NAME = "127.0.0.1";


//--------------------------------------------------------------------
//Purpose:           default constructor.
//Preconditions:    none.
//Postconditions:  a Broker object is instantiated.
//Arguments:       none.
//Returns:           Broker.
//Called Funcs:    increaseCounter().
//--------------------------------------------------------------------
Broker::Broker()
{
    // assign static address and ip.
    serverName = DEFAULT_NAME;
    Broker::increaseCounter();
    return;
}

//--------------------------------------------------------------------
//Purpose:            constructor with a (remote) server name given.
//Preconditions:    must supply a valid server hostname.
//Postconditions:  a Broker object is instantiated with a
//                       remote server hostname.
//Arguments:       string name.
//Returns:           Broker.
//Called Funcs:    increaseCounter().
//--------------------------------------------------------------------
Broker::Broker( const string name )
{
    // assign static address and ip.
    serverName = name;
    Broker::increaseCounter();
    return;
}

//--------------------------------------------------------------------
//Purpose:            default destructor.
//Preconditions:    Broker object.
//Postconditions:  Broker object destroyed.
//Arguments:       none.
//Returns:           0
//Called Funcs:    decreaseCounter().
//--------------------------------------------------------------------
Broker::~Broker()
{
    // remove ourselves from the list.
    Broker::decreaseCounter();
    return;
}

//--------------------------------------------------------------------
//Purpose:            obtain the Broker Counter's current value.
//Preconditions:    none.
//Postconditions:  returns brokerCounter value.
//Arguments:       none.
//Returns:           const int brokerCounter.
//Called Funcs:    none.
//--------------------------------------------------------------------
const int Broker::getCounter()
{
    return brokerCounter;
};

//--------------------------------------------------------------------
//Purpose:            obtain the name of the server which the Broker
//                        is brokering with.
//Preconditions:    none.
//Postconditions:  returns serverName.
//Arguments:       none.
//Returns:           const string serverName.
//Called Funcs:    none.
//--------------------------------------------------------------------
const string Broker::getServerName()
{
    return serverName;
}

//--------------------------------------------------------------------
//Purpose:            prepare for conversion of the provided Document,
//                        and pass to Server for actual converison.
//Preconditions:    Document object.
//Postconditions:  returns true if conversion successfull, false if not.
//                       Converted document location is registered in the
//                       Document.conLocation attribute.
//Arguments:       Document& doc.
//Returns:           bool.
//Called Funcs:   Document.getDocumentConversion(),
//                      getServerName(), Server(),
//                      Server.requestConversion().
//--------------------------------------------------------------------
bool Broker::convertDocument( Document& doc )
{
    // check if conversion is set.
    if ( doc.getDocumentConversion() == "0" )
    {

        // conversion has not been set, error out.
        cout << "Broker Error: no conversion has been assigned to this Document yet...";
        cout << endl;
        return false;
    }

    // need to have a server now.
    Server myConversionServer( serverName );

    if ( !( myConversionServer.requestConversion( doc ) ) )
    {
        cout << "Broker Error: File was not converted by Server..." << endl;
        return false;
        ;
    }
    else
    {
        return true;
    }
}

//--------------------------------------------------------------------
//Purpose:            to increase the brokerCounter attribute.
//Preconditions:    none.
//Postconditions:  brokerCounter attribute has increased by one.
//Arguments:       none.
//Returns:           0.
//Called Funcs:    none.
//--------------------------------------------------------------------
void Broker::increaseCounter()
{
    brokerCounter++;
}

//--------------------------------------------------------------------
//Purpose:            to decrease the brokerCounter attribute.
//Preconditions:    none.
//Postconditions:  brokerCounter attribute has decreased by one.
//Arguments:       none.
//Returns:           0.
//Called Funcs:    none.
//--------------------------------------------------------------------
void Broker::decreaseCounter()
{
    if ( brokerCounter == 1 )
    {
        brokerCounter = 0;
        return;
    }

    brokerCounter--;
    return;
}

//--------------------------------------------------------------------
//Purpose:          obtain list of conversions from Server..
//Preconditions:    Server must be available.
//Postconditions:   returns Result object contianing the list of available
//                        conversions.
//                       This is a 5 column answer:
//                         routine_name,
//                         from_feature_type,
//                         to_feature_type,
//                         from_representation_type,
//                         to_representation_type.
//Arguments:        none.
//Returns:          Result.
//Called Funcs:     Connection(), Transaction(), Result(),
//                        Transaction.Exec(), Transaction.Commit().
//--------------------------------------------------------------------
Result Broker::getConversionListing()
{
    //
    // Just call the server to do the work and assume it works..
    //
    Server myConversionServer( serverName );
    Result myResultsListing = myConversionServer.listConversionDB();
    return myResultsListing;
}


//--------------------------------------------------------------------
//Purpose:          obtain list of available conversions in conversion
//                      database based on given representation type..
//Preconditions:    Server must be available.
//Postconditions:   returns Result object contianing the list of available
//                        conversions.
//                       This is a 5 column answer:
//                         routine_name,
//                         from_feature_type,
//                         to_feature_type,
//                         from_representation_type,
//                         to_representation_type.
//Arguments:        none.
//Returns:          Result.
//Called Funcs:     Broker(), Server.listRepresentaionTypes( string representationType ).
//--------------------------------------------------------------------
Result Broker::getRepresentationTypeListing( string representationType )
{
    //
    // Just call the server to do the work and assume it works..
    //
    Server myConversionServer( serverName );
    Result myRepTypeListing = myConversionServer.listRepesentationTypes( representationType );
    return myRepTypeListing;
}


//--------------------------------------------------------------------
//Purpose:          obtain list of available conversions in conversion
//                      database based on given feature type..
//Preconditions:    Server must be available.
//Postconditions:   returns Result object contianing the list of available
//                        conversions.
//                       This is a 5 column answer:
//                         routine_name,
//                         from_feature_type,
//                         to_feature_type,
//                         from_representation_type,
//                         to_representation_type.
//Arguments:        none.
//Returns:          Result.
//Called Funcs:     Broker(), Server.listFeatureTypes( string featureType ).
//--------------------------------------------------------------------
Result Broker::getFeatureTypeListing( string featureType )
{
    //
    // Just call the server to do the work and assume it works..
    //
    Server myConversionServer( serverName );
    Result myFeatureTypeListing = myConversionServer.listFeatureTypes( featureType );
    return myFeatureTypeListing;
}
