// *******************************************************************
// Server.cpp
// *******************************************************************
//
//  DESCRIPTION: API for Server class. Used by Broker of the
//                        DocConversion project to broker a conversion
//                        of a given document.
//  NOTE:            This program makes use of strings and Server.h.
//
/***************************************************************************
 *   Copyright (C) 2003 by drs. Eric D. Schabell                           *
 *   erics@cs.kun.nl                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <iostream>
#include <string>
#include <fstream>
#include "Server.h"
#include "Document.h"
#include <pqxx/all.h>        // postgresql api.

using namespace std;
using namespace pqxx;

//--------------------------------------------------------------------
//Variables
//--------------------------------------------------------------------
const string DEFAULT_ADDRESS = "127.0.0.1";  // default value.

//--------------------------------------------------------------------
//Purpose:           Default constructor.
//Preconditions:    none.
//Postconditions:  Server object created and exists set to true.
//Arguments:        none.
//Returns:            0.
//Called Funcs:    none.
//--------------------------------------------------------------------
Server::Server()
{
    serverAddress = DEFAULT_ADDRESS;
    return;
}

//--------------------------------------------------------------------
//Purpose:           Constructo with server address.
//Preconditions:    none..
//Postconditions:  Server object created, serverAddress set and
//                        exists set to true.
//Arguments:        string address.
//Returns:            0.
//Called Funcs:    none.
//--------------------------------------------------------------------
Server::Server( string address )
{
    serverAddress = address;
    return;
}

//--------------------------------------------------------------------
//Purpose:           Destructor.
//Preconditions:    Server object exists..
//Postconditions:  Server object destroyed and serverExists set
//                        to false.
//Arguments:        none.
//Returns:            0.
//Called Funcs:    none.
//--------------------------------------------------------------------
Server::~Server()
{
    return;
}

//--------------------------------------------------------------------
//Purpose:            retrieve the Document file from its url location.
//Preconditions:    Document object, either curl or wget application
//                        must be available for downloading file..
//Postconditions:  returns true if file retrieval successfull, false if not.
//                       Only does remote retrieval after checking for local
//                       presence of the file. The location of the file is set
//                       for the Document.fileLocation attribute.
//Arguments:       Document& doc.
//Returns:           bool.
//Called Funcs:   Document.getDocumentFile(), ifstream.open(),
//                      c_str(), system(), Document.getDocumentUrl(),
//                      Document.setFileLocation(), ifstream.close().
//--------------------------------------------------------------------
bool Server::getFile( Document& doc )
{
    string download;  // to be set for curl/wget.

    // Checking if file already exists in /tmp.
    ifstream inputFile;
    string tmpDocFile = "/tmp/" + doc.getDocumentFile();
    inputFile.open( tmpDocFile.c_str() );
    if ( !inputFile )
    {
        // need to pull the file, so setup either curl or wget.
        if ( system( "curl -s http://www.google.com >> /dev/null" ) == 0)
        {
            download = "curl -O ";
        }
        else
        {
            if ( system( "wget -V >> /dev/null  " ) == 0 )
            {
                download = "wget ";
            }
            else
            {
                cout << "Server Error: neither Curl or Wget are operational on this system.";
                cout << endl;
                exit(1);
            }
        }

        // building system call.
        string linuxUrlCall = "cd /tmp; " + download + doc.getDocumentUrl();
        cout << "File is being retrieved..." << endl;
        int results = system( linuxUrlCall.c_str() );
        if ( results > 0 )
        {
            cout << "Server Error: I am unable to retrieve your URL...";
            cout << endl << "Server Error: Please verify the url before trying again.";
            cout << endl;
            doc.setFileLocation( "ErrorURL" );
            return false;
        }
        else
        {
            // determine document name in url, using basename() function.
            string location = "/tmp/" + doc.getDocumentFile();
            doc.setFileLocation( location );
            return true;
        }
    }
    else
    {
        // file exists in /tmp.
        cout << "Server: File has been found locally." << endl;
        inputFile.close();
        string location = "/tmp/" + doc.getDocumentFile();
        doc.setFileLocation( location );
        return true;
    }
}

//--------------------------------------------------------------------
//Purpose:            conversion of the provided Document, based
//                        on the objects registered conversion routine.
//Preconditions:    Document object.
//Postconditions:  returns true if conversion successfull, false if not.
//                       Converted document location is registered in the
//                       Document.conLocation attribute.
//Arguments:       Document& doc.
//Returns:           bool.
//Called Funcs:   Document.getDocumentConversion(),
//                      Document.getFileLocation(), getFile(),
//                      Document.getDocumentFile(),
//                      Document.setFileLocation(),
//                      Document.getDocumentConversion(), system(),
//                      c_str(), Document.setConversionLocation().
//--------------------------------------------------------------------
bool Server::requestConversion( Document& doc )
{
    // check if file has been retrieved before converting.
    if ( doc.getFileLocation() == "0" )
    {
        // file needs to be retrieved first.
        if ( getFile( doc ) )
        {
            string location = "/tmp/" + doc.getDocumentFile();
            doc.setFileLocation( location );
        }
    }

    // attempt to convert file.
    string call = "cd /tmp; " +
                  doc.getDocumentConversion() +
                  " " +
                  doc.getFileLocation();
    int results = system( call.c_str() );

    if ( results >= 0 )
    {
        // conversion successfull!
        doc.setConversionLocation( "/tmp" );
        return true;
    }
    else
    {
        // system call has problems.
        cout << "Server Error: Conversion failed!!!!!" << endl;
        return false;
    }
}


//--------------------------------------------------------------------
//Purpose:          obtain list of conversions in database.
//Preconditions:    DB must be available.
//Postconditions:   returns Result object contianing the list of available
//                        conversions.
//                       This is a 5 column answer:
//                         routine_name,
//                         from_feature_type,
//                         to_feature_type,
//                         from_representation_type,
//                         to_representation_type.
//Arguments:        none.
//Returns:          Result.
//Called Funcs:     Connection(), Transaction(), Result(),
//                        Transaction.Exec(), Transaction.Commit().
//--------------------------------------------------------------------
Result Server::listConversionDB()
{
    try
    {
        // my query.
        string query = "SELECT DISTINCT Filter.routine_name, Filter.from_feature_type, Filter.to_feature_type, ";
        query = query + "Filter.from_representation_type, Filter.to_representation_type FROM Filter, Routine ";
        query = query + "WHERE Filter.routine_name = Routine.name;";

        // setup connection.
        Connection myConnect( "dbname=clearinghouse user=pronir" );

        // Begin a transaction acting on our current connection.
        Transaction myTrans(myConnect, "listingConversions");

        // Perform a query on the database, storing result tuples in R
        Result myResult = myTrans.Exec( query.c_str() );

        // Tell transation that it was successful.
        myTrans.Commit();

        // return the results object.
        return myResult;
    }
    catch ( const exception &e)
    {
        // All exceptions thrown by libpqxx are derived from std::exception.
        cerr << "Exception: " << e.what() << endl;
        exit(2);
    }
    catch (...)
    {
        // This is really unexpected (see above).
        cerr << "Unhandled exception..." << endl;
        exit(100);
    }
}


//--------------------------------------------------------------------
//Purpose:          obtain list of conversions in database based on
//                      given representation type.
//Preconditions:   DB must be available.
//Postconditions: returns Result object contianing the list of available
//                      conversions.
//                      This is a 5 column answer:
//                      routine_name,
//                      from_feature_type,
//                      to_feature_type,
//                      from_representation_type,
//                      to_representation_type.
//Arguments:       none.
//Returns:           Result.
//Called Funcs:    Connection(), Transaction(), Result(),
//                       Transaction.Exec(), Transaction.Commit().
//--------------------------------------------------------------------
Result Server::listRepesentationTypes( string representationType )
{
    try
    {
        //
        // my query, careful with the spacing...
        //
        string query = "SELECT DISTINCT Filter.routine_name, Filter.from_feature_type, Filter.to_feature_type, ";
        query = query + "Filter.from_representation_type, Filter.to_representation_type FROM Filter, Routine ";
        query = query + "WHERE Filter.from_representation_type='";
        query = query + representationType + "' ";
//        query = query + "OR Filter.from_representation_type='*' ";
        query = query + "OR Filter.to_representation_type='";
        query = query + representationType + "' ";
				query = query + ";";
//        query = query + "OR Filter.to_representation_type='*'; ";

        // setup connection.
        Connection myConnect( "dbname=clearinghouse user=pronir" );

        // Begin a transaction acting on our current connection.
        Transaction myTrans(myConnect, "listingRepresentationTypes");

        // Perform a query on the database, storing result tuples in R
        Result myResult = myTrans.Exec( query.c_str() );

        // Tell transation that it was successful.
        myTrans.Commit();

        // return the results object.
        return myResult;
    }
    catch ( const exception &e)
    {
        // All exceptions thrown by libpqxx are derived from std::exception.
        cerr << "Exception: " << e.what() << endl;
        exit(2);
    }
    catch (...)
    {
        // This is really unexpected (see above).
        cerr << "Unhandled exception..." << endl;
        exit(100);
    }
}


//--------------------------------------------------------------------
//Purpose:          obtain list of conversions in database based on
//                      given feature type.
//Preconditions:   DB must be available.
//Postconditions: returns Result object contianing the list of available
//                      conversions.
//                      This is a 5 column answer:
//                      routine_name,
//                      from_feature_type,
//                      to_feature_type,
//                      from_representation_type,
//                      to_representation_type.
//Arguments:       none.
//Returns:           Result.
//Called Funcs:    Connection(), Transaction(), Result(),
//                       Transaction.Exec(), Transaction.Commit().
//--------------------------------------------------------------------
Result Server::listFeatureTypes( string featureType )
{
    try
    {
        //
        // my query, careful with the spacing...
        //
        string query = "SELECT DISTINCT Filter.routine_name, Filter.from_feature_type, Filter.to_feature_type, ";
        query = query + "Filter.from_representation_type, Filter.to_representation_type FROM Filter, Routine ";
        query = query + "WHERE Filter.from_feature_type='";
        query = query + featureType + "' ";
//      query = query + "OR Filter.from_feature_type='*' ";
        query = query + "OR Filter.to_feature_type='";
        query = query + featureType + "' ";
				query = query + ";";
//      query = query + "OR Filter.to_feature_type='*'; ";

        // setup connection.
        Connection myConnect( "dbname=clearinghouse user=pronir" );

        // Begin a transaction acting on our current connection.
        Transaction myTrans(myConnect, "listingFeatureTypes");

        // Perform a query on the database, storing result tuples in R
        Result myResult = myTrans.Exec( query.c_str() );

        // Tell transation that it was successful.
        myTrans.Commit();

        // return the results object.
        return myResult;
    }
    catch ( const exception &e)
    {
        // All exceptions thrown by libpqxx are derived from std::exception.
        cerr << "Exception: " << e.what() << endl;
        exit(2);
    }
    catch (...)
    {
        // This is really unexpected (see above).
        cerr << "Unhandled exception..." << endl;
        exit(100);
    }
}
