// *******************************************************************
// docConversionTest.cpp
// *******************************************************************
//
//  DESCRIPTION: This is a unit testing application to test the
//               DocConvert application. It makes use of the C++
//               Unit++ testing framework.
//  NOTE:        This program makes use of the Unit++ testing suite
//               Document.h, Broker.h and Server.h.
//
/***************************************************************************
*   Copyright (C) 2003 by drs. Eric D. Schabell                           *
*   erics@cs.kun.nl                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <unit++.h>     // unit testing with Unit++.
#include <string>       // STL strings.
#include <pqxx/all.h>   // postgresql api.
#include "Document.h"
#include "Broker.h"
#include "Server.h"

using namespace std;
using namespace unitpp;

namespace
{
class DocConversionTestSuite : public suite
{
    //--------------------------------------------------------------------
    //Purpose:           Test the construct of remote Server request.
    //Preconditions:    none.
    //Postconditions:  server name checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:    Broker(), Broker.getServerName(), assert_true().
    //--------------------------------------------------------------------
    void testBrokerRemoteServer()
    {
        Broker myBroker( "my.conversion.server" );
        string myServerName = myBroker.getServerName();
        if ( myServerName == "my.conversion.server" )
        {
            assert_true( "broker remote server check", true );
        }
        else
        {
            assert_true( "broker remote server check", false );
        }
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the getCounter method.
    //Preconditions:    Broker object.
    //Postconditions:  Broker counter is checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:    Broker(), Broker.getCounter(), assert_eq().
    //--------------------------------------------------------------------
    void testGetCounter()
    {
        Broker myBroker;
        assert_eq("broker counter check", 1, myBroker.getCounter() );
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the increaseCounter method.
    //Preconditions:    Two Broker objects.
    //Postconditions:  Broker counter is increased and checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:    Broker(), Broker.getCounter(), assert_eq().
    //--------------------------------------------------------------------
    void testIncreaseCounter()
    {
        Broker myBroker;
        assert_eq("initial counter check", 1, myBroker.getCounter() );
        Broker anotherBroker;
        assert_eq("increase counter check", 2, anotherBroker.getCounter() );
    }

    //--------------------------------------------------------------------
    //Purpose:           Test decresing of Broker Counter.
    //Preconditions:    Two Broker objects.
    //Postconditions:  Broker counter is increased, decreased and checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:    Broker(), ~Broker(), Broker.getCounter(), assert_eq().
    //--------------------------------------------------------------------
    void testDecreaseCounter()
    {
        {
            Broker myBroker;
            Broker anotherBroker;
            int mycounter = anotherBroker.getCounter();
            assert_eq("pre decrease counter check", 2, mycounter );
        }
        Broker yetAnotherBroker;
        int yetanothercounter = yetAnotherBroker.getCounter();
        assert_eq("decreased counter check", 1, yetanothercounter );
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the getDocumentUrl method.
    //Preconditions:   Document object.
    //Postconditions: Document getDocumentUrl method checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:   Document( URI, routine ), Document.getDocumentUrl(),
    //                      assert_eq().
    //--------------------------------------------------------------------
    void testGetDocumentUrl()
    {
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        string myUrl = myDocument.getDocumentUrl();
        int match;  // to check matching strings.
        if ( myUrl == "http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse .pdf" )
        {
            match = 0;
        }
        else
        {
            match = 1;
        }
        assert_eq("get Document url check", 0, match);
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the getDocumentFile method.
    //Preconditions:   Document object.
    //Postconditions: Document getDocumentFile method checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:   Document( URI, routine ), Document.getDocumentFile(),
    //                      assert_eq().
    //--------------------------------------------------------------------
    void testGetDocumentFile()
    {
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        string myFile = myDocument.getDocumentFile();
        int match;  // to check matching strings.
        if ( myFile == "tech_clearinghouse.pdf" )
        {
            match = 0;
        }
        else
        {
            match = 1;
        }
        assert_eq("get Document file check", 0, match);
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the getDocumentConversion method.
    //Preconditions:   Document object.
    //Postconditions: Document getDocumentConversion method checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:   Document( URI, routine ),
    //                     Document.getDocumentConversion(), assert_eq().
    //--------------------------------------------------------------------
    void testGetDocumentConversion()
    {
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        string myConversion = myDocument.getDocumentConversion();
        int match;  // to check matching strings.
        if ( myConversion == "pdf2ps" )
        {
            match = 0;
        }
        else
        {
            match = 1;
        }
        assert_eq("get Document conversion check", 0, match);
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the getConversionLocation method.
    //Preconditions:   Document object.
    //Postconditions: Document getConversionLocation method checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:   Document( URI, routine ),
    //                     Document.setConversionLocation(),
    //                     Document.getConversionLocation(), assert_eq().
    //--------------------------------------------------------------------
    void testGetDocumentConLocation()
    {
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        myDocument.setConversionLocation("http://localhost/conversions");
        string loc = myDocument.getConversionLocation();
        int match;  // to check matching strings.
        if ( loc == "http://localhost/conversions" )
        {
            match = 0;
        }
        else
        {
            match = 1;
        }
        assert_eq("get Document conversion location check", 0, match);
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the getFileLocation method.
    //Preconditions:   Document object.
    //Postconditions: Document getFileLocation method checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:   Document( URI, routine ),
    //                     Document.setFileLocation(),
    //                     Document.getFileLocation(), assert_eq().
    //--------------------------------------------------------------------
    void testGetDocumentFileLocation()
    {
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        myDocument.setFileLocation("/tmp");
        string myFileLocation = myDocument.getFileLocation();
        int match;  // to check matching strings.
        if ( myFileLocation == "/tmp" )
        {
            match = 0;
        }
        else
        {
            match = 1;
        }
        assert_eq("get Document file location check", 0, match);
    }

    //--------------------------------------------------------------------
    //Purpose:           Test the convertDocument method.
    //Preconditions:   Broker and Document objects.
    //Postconditions: Document convertDocument method checked.
    //Arguments:       none.
    //Returns:           0.
    //Called Funcs:   Document( URI, routine ), Broker(),
    //                     Broker.convertDocument(), assert_true().
    //--------------------------------------------------------------------
    void testConvertDocument()
    {
        Broker myBroker;
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        if ( myBroker.convertDocument(myDocument) )
        {
            assert_true( "Conversion results", true );
        }
        else
        {
            assert_true( "Conversion results", false );
        }
    }

    //--------------------------------------------------------------------
    //Purpose:          Test Server requestConversion method.
    //Preconditions:    Supply a correctly instantiated Document.
    //Postconditions:   Returns true if conversion successful, false
    //                  if not.
    //Arguments:        none.
    //Returns:          0.
    //Called Funcs:     Server(), Document(), Server.requestConversion().
    //--------------------------------------------------------------------
    void testServerRequestConversion()
    {
        Server myServer;
        Document
        myDocument("http://infolab.uvt.nl/people/erics/papers/tech-clearinghouse/tech_clearinghouse.pdf", "pdf2ps");
        if ( myServer.requestConversion( myDocument ) )
        {
            assert_true( "server request conversion check", true );
        }
        else
        {
            assert_true( "server request conversion check", false );
        }
    }

    //--------------------------------------------------------------------
    //Purpose:          Test Broker/Server listConversionDB  method.
    //Preconditions:    Supply Broker.
    //Postconditions:   A complete listing of all available converison
    //                  routines available in the Clearinghouse DB.
    //Arguments:        none.
    //Returns:          0.
    //Called Funcs:     Broker(), Broker.listConversionDB().
    //--------------------------------------------------------------------
    void testListConversionDB()
    {
        Broker myBroker;
        Result myDBQuery = myBroker.getConversionListing();
        if ( myDBQuery.size() > 0 )
        {
            assert_true( "list conversions check", true );
        }
        else
        {
            assert_true( "list conversions check", false );
        }
    }


    //--------------------------------------------------------------------
    //Purpose:           Test Broker/Server Representaion type query method.
    //Preconditions:    Supply Broker.
    //Postconditions:  A listing of all available converison
    //                      routines based on given representation type in
    //                     the Clearinghouse DB (both from and to).
    //Arguments:      none.
    //Returns:          0.
    //Called Funcs:   Broker(), Broker.getRepresentaionTypeListing( string repType).
    //--------------------------------------------------------------------
    void
    testServerRepresentationTypeQuery()
    {
        Broker myBroker;
        string representationTypeTester = "pdf";

        Result myRepresentationQuery =
            myBroker.getRepresentationTypeListing( representationTypeTester );
        if ( myRepresentationQuery.size() > 0 )
        {
            assert_true( "representation types listing", true );
        }
        else
        {
            assert_true( "representation types listing", false );
        }
    }


    //--------------------------------------------------------------------
    //Purpose:           Test Broker/Server feature type query method.
    //Preconditions:  Supply Broker.
    //Postconditions: A listing of all available converison routines
    //                          based on given feature type in the Clearinghouse
    //                          DB (both from and to).
    //Arguments:       none.
    //Returns:            0.
    //Called Funcs:   Broker(), Broker.getFeatureTypeListing( string featureType).
    //--------------------------------------------------------------------
    void
    testServerFeatureTypeQuery()
    {
        Broker myBroker;
        string featureTypeTester = "binary";

        Result myFeatureQuery =
            myBroker.getFeatureTypeListing( featureTypeTester );
        if ( myFeatureQuery.size() > 0 )
        {
            assert_true( "feature types listing", true );
        }
        else
        {
            assert_true( "feature types listing", false );
        }
    }


public:
    DocConversionTestSuite() : suite("DocConversionTestSuite")
    {
        add
            ( "BrokerRemoteServer",
                    testcase( this, "testBrokerRemoteServer",
                              &DocConversionTestSuite::testBrokerRemoteServer )
            );
        add
            ( "BrokerListConversionDB",
                    testcase( this, "testListConversionDB",
                              &DocConversionTestSuite::testListConversionDB )
            );
        add
            ( "DocumentGetDocumentConLocation",
                    testcase( this, "testGetDocumentConLocation",

                              &DocConversionTestSuite::testGetDocumentConLocation )            );
        add
            ( "BrokerDecreaseCounter",
                    testcase( this, "testDecreaseCounter",
                              &DocConversionTestSuite::testDecreaseCounter )
            );
        add
            ( "BrokerIncreaseCounter",
                    testcase( this, "testIncreaseCounter",
                              &DocConversionTestSuite::testIncreaseCounter )
            );
        add
            ( "BrokerGetCounter",
                    testcase( this, "testGetCounter",
                              &DocConversionTestSuite::testGetCounter )
            );
        add
            ( "DocumentGetDocumentUrl",
                    testcase( this, "testGetDocumentUrl",
                              &DocConversionTestSuite::testGetDocumentUrl )
            );
        add
            ( "DocumentGetDocumentFile",
                    testcase( this, "testGetDocumentFile",
                              &DocConversionTestSuite::testGetDocumentFile )
            );
        add
            ( "DocumentGetDocumentConversion",
                    testcase( this, "testGetDocumentConversion",
                              &DocConversionTestSuite::testGetDocumentConversion
                            )            );
        add
            ( "DocumentGetDocumentFileLocation",
                    testcase( this, "testGetDocumentFileLocation",

                              &DocConversionTestSuite::testGetDocumentFileLocation )            );
        add
            ( "DocumentConvertDocument",
                    testcase( this, "testConvertDocument",
                              &DocConversionTestSuite::testConvertDocument )
            );
        add
            ( "ServerRequestConversion",
                    testcase( this, "testServerRequestConversion",
                              &DocConversionTestSuite::testServerRequestConversion )            );
        add
            ( "ServerRequestRepresentationTypeQuery",
                    testcase( this, "testServerRepresentationTypeQuery",
                              &DocConversionTestSuite::testServerRepresentationTypeQuery )            );
        add
            ( "ServerRequestFeatureTypeQuery",
                    testcase( this, "testServerFeatureTypeQuery",
                              &DocConversionTestSuite::testServerFeatureTypeQuery )            );
        suite::main().add("DocConversionTestSuite",this);
    }

};

DocConversionTestSuite* theDocConversionsTest = new DocConversionTestSuite();
}

