// *******************************************************************
// main.cpp
// *******************************************************************
//
//  DESCRIPTION: Client application that provides a command line
//                         interface to the DocConversion project. With
//                        'docconversion' the user can convert documents
//                         from anywhere on the Internet.
//  NOTE:             This program makes use of Broker.h, Document.h,
//                         strings and system() system call.
//
/***************************************************************************
*   Copyright (C) 2003 by drs. Eric D. Schabell                           *
*   erics@cs.kun.nl                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

//--------------------------------------------------------------------
//#includes
//--------------------------------------------------------------------
#include <iostream>
#include <string>
#include <cstdlib>
#include <pqxx/all.h>
#include "Broker.h"
#include "Document.h"

using namespace std;
using namespace pqxx;


//--------------------------------------------------------------------
// Variables
//--------------------------------------------------------------------
static const int MIN_ARGS = 2;    // minimum args needed to run this stuff.
enum Flag {                             // handles agrs passed.
    listingQuery,
    remoteServer,
    representaionTypeQuery,
    featureTypeQuery,
    none
};


//--------------------------------------------------------------------
//Purpose:           Display usage information.
//Preconditions:    none.
//Postconditions:  Usage information is displayed.
//Arguments:       none.
//Returns:           0.
//Called Funcs:    none.
//--------------------------------------------------------------------
void usage()
{
    cout << endl;
    cout << "Usage: " << endl;
    cout << "docconversion [options] <conversion> <document URI>" << endl;
    cout << "docconverison -l" << endl;
    cout << endl;
    cout << "Options:";
    cout << endl;
    cout << "-l             = show list of available converison routines from ";
    cout << endl;
    cout << "                 the Clearinghouse database.";
    cout << endl << endl;
    cout << "-r <representation-type>  = show list of available conversions ";
    cout << "based " << endl;
    cout << "                 on the supplied representaiton type (such as ";
    cout << "pdf, html, text, gz, *)." << endl;
    cout << "                 This will show all conversion routines where ";
    cout << "the supplied type" << endl;
    cout << "                 is available as a conversion'from' this type, ";
    cout << "or as a conversion" << endl;
    cout << "                 'to' this type.";
    cout << endl << endl;
    cout << "-s <server>    = hostname of converison server to be used.";
    cout << endl << "                 Default is localhost conversions.";
    cout << endl << endl;
    cout << "<conversion>   = name of conversion ";
    cout << "routine to be used.";
    cout << endl << endl;
    cout << "<document URI> = location of document to be converted {html|ftp}.";
    cout << endl << endl;
    cout << "Note: to check all possible feature|representation types you can pass the '*' (in quotes).";
    cout << endl << endl;
    return;
}


//--------------------------------------------------------------------
//Purpose:           Determine type of args passed.
//Preconditions:   none.
//Postconditions: returns an enum value Flag.
//Arguments:       string myArg.
//Returns:           enum Flag if args correct, otherwise fails with
//                      display of usage() function.
//Called Funcs:   usage().
//--------------------------------------------------------------------
Flag determineArgs( string myArg )
{
    Flag myFlag;

    // check for listing query request.
    if ( myArg == "-l" )
    {
        return ( myFlag = listingQuery );
    }
    else
    {
        // check for representation type query request.
        if ( myArg == "-r" )
        {
            return ( myFlag = representaionTypeQuery );
        }
    }

    // check if remote server requested.
    if ( myArg == "-s" )
    {
        return ( myFlag = remoteServer );
    }
    else
    {
        // check for feature type query request.
        if ( myArg == "-f" )
        {
            return ( myFlag = featureTypeQuery );
        }
        else
        {
            return ( myFlag = none );
        }
    }
}


//--------------------------------------------------------------------
//Purpose:           Displays the query results.
//Preconditions:   given a vaild query.
//Postconditions: query has been displayed.
//Arguments:       Result query.
//Returns:           void.
//Called Funcs:   none.
//--------------------------------------------------------------------
void displayQueryResults( Result query )
{
    int tabsize = 4;  // for report layout.

    // Process each successive result tuple with this loop,
    // remember there are 5 columns.
    cout << "Routine Name";
    cout << '\t' << '\t' << "From Feature";
    cout << '\t' << "From Representation";
    cout << '\t' << "To Feature";
    cout << '\t' << "To Representation";
    cout << endl;
    cout << "===========";
    cout << '\t' << '\t' << "===========";
    cout << '\t' << "================";
    cout << '\t' << "===========";
    cout << '\t' << "==============" << endl;

    //    for (Result::const_iterator c = R.begin(); c != R.end(); ++c)
    for ( Result::size_type i = 0; i != query.size(); ++i )
    {
        // Output column elements.
        cout << query[i]["routine_name"];
        cout << '\t' << '\t';

        // check for formating text, the size of next element.
        if ( query[i]["routine_name"].size() < ( tabsize + tabsize ) )
        {
            // small, need extra tab.
            cout << '\t';
        }
        cout << query[i]["from_feature_type"];
        cout << '\t';

        // check for formating text, the size of next element.
        if ( query[i]["from_feature_type"].size() < ( tabsize + tabsize ) )
        {
            // small, need extra tab.
            cout << '\t';
        }
        cout << query[i]["from_representation_type"];
        cout << '\t' << '\t';

        // check for formating text, the size of next element.
        if ( query[i]["from_representation_type"].size() < ( tabsize + tabsize )
           )
        {
            // small, need extra tab.
            cout << '\t';
        }
        cout << query[i]["to_feature_type"];
        cout << '\t';

        // check for formating text, the size of next element.
        if ( query[i]["to_feature_type"].size() < ( tabsize + tabsize ) )
        {
            // small, need extra tab.
            cout << '\t';
        }
        cout << query[i]["to_representation_type"];
        cout << endl;
    }
}



//--------------------------------------------------------------------
//Purpose:           Client interface to DocConvert program.
//Preconditions:   User inputs 'docconvert' and minimum of two
//                       arguments, a conversion routine name and
//                       a document URI to be converted.
//Postconditions:  User is provided with a converted document
//                       based on her requested conversion routine,
//                       or error information.
//Arguments:       int argc, char *const argv[].
//Returns:           0.
//Called Funcs:    usage(), system(), c_str(), Document( URI, routine ),
//                      Document.getConversionLocation(),
//                      Document.getDocumentFile(),
//                      Document.getDocumentConversion(),
//                      Broker( server ), Broker(),
//                      Broker.convertDocument( Document ).
//--------------------------------------------------------------------
int main( int argc, char *const argv[] )
{
    // Check for no args.
    if ( argc < MIN_ARGS )
    {
        cout << "Wrong usage of docconvert options." << endl;
        usage();
        exit(1);
    }

    // determine which flags if any.
    --argc;
    ++argv;
    string argument = argv[0];
    Flag myFlag = determineArgs( argument );

    switch( myFlag )
    {

    case listingQuery:

        if ( argc != 1 )
        {
            // means more options after flag.
            usage();
            exit(1);
        }
        else
        {
            // no more options, so get listing.
            Broker myBroker;
            Result myResult = myBroker.getConversionListing();
            if ( myResult.size() > 0 )
            {
                cout << endl << endl;
                cout << "A listing of all possible conversions provided by the ";
                cout << "DocConversion application:";
                cout << endl << endl;

                displayQueryResults( myResult );
            }
            else
            {
                cout << "Something wrong with -l listing query results...";
                cout << endl;
                exit(1);
            }
        }
        // done here so bail.
        break;

    case remoteServer:

        if ( argc != ( MIN_ARGS * 2 ) )
        {
            usage();
            exit(1);
        }
        else
        {
            // extract the server name, conversion and document.
            ++argv;
            string myserver = argv[0];
            ++argv;
            string myconversion = argv[0];
            ++argv;
            string mydocuri = argv[0];

            // create our Document.
            Document myDoc( mydocuri, myconversion );

            // inform user.
            cout << "Converting document: " << endl;
            cout << "   " << myDoc.getDocumentFile() << endl;
            cout << "From URI: " << endl;
            cout << "   " << mydocuri << endl;
            cout << "Using conversion routine:" << endl;
            cout << "   " << myDoc.getDocumentConversion();
            cout << endl << endl;

            // setup remote Broker and request conversion.
            Broker docTransformer( myserver );

            if ( docTransformer.convertDocument( myDoc ) )
            {
                cout << "Conversion is available somewhere in ";
                cout << myDoc.getConversionLocation();
                cout << ":" << endl;

                // setup system call for displaying conversion results dir.
                string call = "cd " + myDoc.getConversionLocation() +
                              "; " + " ls -lhd /tmp/*";
                system( call.c_str() );
                cout << endl;
                cout << "Thank you for using DocConversion!" << endl;
            }
            else
            {
                cout << "Conversion was not possible..." << endl;
                cout << "Maybe a bad document URI, wanna take a look ";
                cout << "at usage?" << endl;
                usage();
                exit( 1 );
            }
        }
        // done here so bail.
        break;

    case representaionTypeQuery:

        if ( argc != MIN_ARGS )
        {
            // means more args after passed parameter.
            usage();
            exit(1);
        }
        else
        {
            ++argv;
            string requestedRepType = argv[0];
            Broker myBroker;
            Result myRepResult =
                myBroker.getRepresentationTypeListing( requestedRepType );
            if ( myRepResult.size() > 0 )
            {
                cout << endl << endl;
                cout << "Based on the given representation type, the following routines ";
                cout << " are provided by the DocConversion application:";
                cout << endl << endl;
                displayQueryResults( myRepResult );
            }
            else
            {
                cout << "The representation type you requested is not available on our ";
                cout << endl;
                cout << "system... This could mean that the database is unavailable at ";
                cout << endl;
                cout << "this time, or that the feature type is really not provided by ";
                cout << endl;
                cout << "our system.";
                cout << endl;
            }
        }
        // done here so bail.
        break;

    case featureTypeQuery:

        if ( argc != MIN_ARGS )
        {
            // means more args after passed parameter.
            usage();
            exit(1);
        }
        else
        {
            ++argv;
            // passed feature type parameter.
            string requestedFeatureType = argv[0];
            Broker myBroker;
            Result myFeatureResult =
                myBroker.getFeatureTypeListing( requestedFeatureType );
            if ( myFeatureResult.size() > 0 )
            {
                cout << endl << endl;
                cout << "Based on the given feature type, the following routines ";
                cout << " are provided by the DocConversion application:";
                cout << endl << endl;
                displayQueryResults( myFeatureResult );
            }
            else
            {
                cout << "The feature type you requested is not available on our system... ";
                cout << endl;
                cout << "This could mean that the database is unavailable at this time, ";
                cout << endl;
                cout << "or that the feature type is really not provided by our system.";
                cout << endl;
            }
        }
        // done here so bail.
        break;

    case none:

        // no options, so  deal with document conversion.
        if ( argc != MIN_ARGS )
        {
            usage();
            exit(1);
        }
        else
        {
            // extract the conversion and document
            string myconversion = argv[0];
            ++argv;
            string mydocuri = argv[0];

            // create our Document.
            Document myDoc( mydocuri, myconversion );

            // inform user.
            cout << "Converting document: " << endl;
            cout << "   " << myDoc.getDocumentFile() << endl;
            cout << "From URI: " << endl;
            cout << "   " << mydocuri << endl;
            cout << "Using conversion routine:" << endl;
            cout << "   " << myDoc.getDocumentConversion();
            cout << endl << endl;

            // setup default Broker and request conversion.
            Broker docTransformer;
            if ( docTransformer.convertDocument( myDoc ) )
            {
                cout << "Conversion is available somewhere in " <<
                myDoc.getConversionLocation();
                cout << ":" << endl;

                // setup system call for displaying conversion results dir.
                string call = "cd " + myDoc.getConversionLocation() + "; " +
                              " ls -lhd /tmp/*";
                system( call.c_str() );
                cout << endl;
                cout << "Thank you for using DocConversion!" << endl;
            }
            else
            {
                cout << "Conversion was not possible..." << endl;
                cout << "Maybe a bad document URI, wanna take a look at usage?" << endl;
                usage();
                exit( 1 );
            }
        }
        // done here so bail.
        break;
    } // end switch.

    //
    // Say goodbye!
    //
    cout << endl << endl;
    cout << "Thank you for allowing DocConversion to be your converison ";
    cout << endl;
    cout << "service provider. We look forward to helping you again in ";
    cout << endl;
    cout << "the near future.";
    cout << endl;
    cout << "                           'The DocConversion Team'" << endl;
}
